SUBROUTINE INIT
!===========================================================================
!
!          *** INITIALZE CONSTANTS ADN DEFAULT VALUES ***
!           Time-stamp: < 2011/11/10 17:12  T.IKARI >
!
!  * initialize parameters
!  * read input namelist
!===========================================================================
  USE COMM
  IMPLICIT NONE

  
!==========================================
!    RR    :  MAJOR RADUIS
!    RA    :  MINOR RADIUS
!==========================================
  
  RR   = 2.D0
  RA   = 1.D0
  RKAP = 1.D0
  RDLT = 1.D0
  BB   = 1.D0

!===============================================
!    NRMAX    :  NUMBER OF RADIAL NODAL POINTS
!    NTMAX    :  NUMBER OF TIME STEP
!    NITMAX   :  MAZIMUM NUMBER OF ITERATION
!    NSMAX    :  NUMBER OF PARTICLE SPECIES
!    DT       :  SIZE OF TIME STEP
!    EPSIT    :  CONVERGENCE CRITERION
!===============================================

  NRMAX  = 50
  NTMAX  = 100
  NITMAX = 100
  NSMAX  = 1
  DT     = 0.01D0
  EPSIT  = 1.D-8

!===============================================
!    MDLM = 0  :  EQUALLY-SPACED MESH POINTS
!    MDLM = 1  :  ARBITRARY ( READ DATA FILE )
!===============================================

  MDLM = 0

!===============================================
!    T0  : INITIAL CENTRAL TEMPERATURE
!    TA  : INITIAL EDGE TEMPERATURE
!    PW0 : HEATING POWER AT THE CENTER
!    PWA : HEATING POWER AT THE EDGE
!===============================================

  T0  = 1.D0
  TA  = 0.1D0
  PW0 = 0.1D0
  PWA = 0.1D0

!===========================================================
!    MDLD = 0 : 'DG' : CONSTANT
!    MDLD = 1 : 'DG' : STEP FUNCTION (Tokar)
!    MDLD = 2 : 'DG' : STIFF MODEL (Pereverzev)
!    MDLD = 3 : 'DG' : ROW AT EDGE REGION (STATIONARY)
!
!    MDLDB = 0 : D_bar IS CONSTANT
!    MDLDB = 1 : D_bar IS CALCULATED ON EVERY GRID POINTS
!                                         AT EVERY TIME STEP
!===========================================================

  MDLD  = 0
  MDLP  = 0
  MDLDB = 0
  ! D0 < D1
  D0   = 0.01D0
  D1   = 0.1D0
  DBF  = D1

  LTCR = 1.D0
!  DBFACT = 1.D0

END SUBROUTINE INIT


SUBROUTINE PARM
  USE COMM
  IMPLICIT NONE

!  CHARACTER(70) :: LINE
!  INTEGER(4) :: KL1

  NAMELIST /in/ NRMAX, NTMAX, NSMAX, NITMAX, EPSIT, DT, T0, TA,  &
                RR, RA, RKAP, RDLT, BB,  &
                PW0, PWA, D0, D1, &
                MDLM, MDLD, MDLP, MDLDB, DBF

1 CONTINUE

  INPUTEND = 0

  WRITE(6,*) '========== THE VALUE OF PARAMETERS =========='
  WRITE(6,601) 'NRMAX = ',NRMAX,   'NTMAX = ',NTMAX
  WRITE(6,601) 'NSMAX = ',NSMAX,   'NITMAX= ',NITMAX
  WRITE(6,602) 'EPSIT = ',EPSIT,   'DT    = ',DT
  WRITE(6,602) 'T0    = ',T0,      'TA    = ',TA
  WRITE(6,602) 'PW0   = ',PW0,     'PWA   = ',PWA
  WRITE(6,602) 'D0    = ',D0,      'D1    = ',D1
  WRITE(6,601) 'MDLM  = ',MDLM,    'MDLD  = ',MDLD
  WRITE(6,603) 'DBF   = ',DBF,     'MDLDB = ',MDLDB
  WRITE(6,*) '============================================='
  WRITE(6,*) ' '

601 FORMAT(A8,I8,6X,A8,I8)
602 FORMAT(A8,1PE12.4,2X,A8,1PE12.4)
603 FORMAT(A8,1PE12.4,2X,A8,I8)

  WRITE(6,*) 'INPUT name=value, ^D FOR QUIT:'
  READ(5,'(A80)',END=1000,ERR=1) LINE
  CALL KTRIM(LINE,KL1)
  WRITE(6,'(A)')' $in '//LINE(1:KL1)//' $END'
  WRITE(7,'(A)')' $in '//LINE(1:KL1)//' $END'
  REWIND(7)
  READ(7,in,ERR=1)
  REWIND(7)

RETURN
1000 INPUTEND = 1
END SUBROUTINE PARM


SUBROUTINE MESH_GEN
  USE COMM

 INTEGER(4) :: NR

  SELECT CASE(MDLM)
  !--- EQUALLY-SPACED MESH POINTS ---
  CASE(0) 
     RG(1) = 0.D0
     DO NR = 2, NRMAX+1
        DR(NR-1) = RA/DBLE(NRMAX)
        RG(NR) = RG(NR-1) + DR(NR-1)
     END DO
  END SELECT

RETURN

END SUBROUTINE MESH_GEN
